/* lw-support/src/tests/Net_SMTPClient.cpp
 *
 *  (c)2005, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

#include "lw-support"

#include <iostream>



int main(int argc, char* argv[])
{
    if(argc == 2 && !strcmp(argv[1], "--print-summary")) {
        std::wcout << L"Tests the simple SMTP client.\n";
        return 0;
    }

    if(argc != 5) {
        std::wcout << L"Expecting arguments:\n"
                      L"  SMTP server hostname\n"
                      L"  SMTP server port\n"
                      L"  mail from\n"
                      L"  rcpt to\n"
                      L"and then pipe data into stdin.\n";
        return 1;
    }

    // set up logs
    lw::Log logger(std::wcout, true);
    lw::Log logError(std::wcerr, true);

    int ret = 0;
    try {
        // process arguments
        std::string hostName = argv[1];
        lw::NetAddressIPv4 hostAddr = lw::NetAddressIPv4::fromDNS(hostName);
        uint16_t port = lw::strToUInt16(lw::strToUcs4(argv[2]));
        std::string mailFrom = argv[3];
        std::string rcptTo = argv[4];

        logger << lw::Log::start(L"DNS")
               << L"Resolved host."
                  L"\n  Host name   : " << hostName
               << L"\n  IPv4 Address: " << (lw::NetAddress&)(hostAddr)
               << lw::Log::end;

        // read input
        std::string data, line;
        while(std::cin) {
            getline(std::cin, line);
            data.append(line);
            data.append("\n");
        }

        logger << lw::Log::start(L"Program")
               << L"Sending " << data.size()
               << L" bytes of data."
               << lw::Log::end;

        // prepare header lines
        std::multimap<std::string, std::string> mailHeaders;
        mailHeaders.insert(make_pair(std::string("subject"),
            std::string("lw-support test message")));

        // instantiate event manager
        lw::EventManager eventManager(1, 1);
        lw::CompletionNotifier notify;

        // send email
        lw::LoggingSMTPClient::sendMail(
            hostAddr, port, eventManager, "localhost",
            mailFrom, rcptTo, mailHeaders, data,
            logger, logError, &notify);

        while(true) {
            try {
                notify.wait(0);
                break;
            }
            catch(lw::CompletionNotifier::Timeout&) {
                eventManager.processEvents(-1);
                continue;
            }
        }

        logger << lw::Log::start(L"Program")
               << L"Process completed successfully."
               << lw::Log::end;
    }
    catch(lw::Exception& e) {
        logError << lw::Log::start(L"Program")
                 << e
                 << lw::Log::end;
        ret = 1;
    }
    catch(...) {
        logError << lw::Log::start(L"Program")
                 << L"Unknown exception caught.\n"
                 << lw::Log::end;
        ret = 1;
    }

    return ret;
}
